function dict = vl_fisher_encoder_train(X, Y, N_PATCH_SIZE, N_DICT_SIZE, useDiffSeq)

if ~exist('useDiffSeq', 'var')
  useDiffSeq = false;
end

N_INSTANCE = size(X,1);
N_CLASS = length(unique(Y));

t0 = tic;
[mappedX, mappedY, ~] = extract_patches(N_PATCH_SIZE, X, Y);
fprintf('[extract_patches] instance %d time %.2f\n', size(Y,1), toc(t0));

N_PATCHES = size(mappedX,1);

if useDiffSeq
  mappedX1 = transform_vector(mappedX,'abs_diff');
  % mappedX1 = abs(mappedX1); % 777
else
  mappedX1 = transform_vector(mappedX,'abs');
end

numClusters = N_DICT_SIZE;
[means1, covariances1, priors1] = vl_gmm(mappedX1', numClusters, 'verbose');

if useDiffSeq
  mappedX2 = transform_vector(mappedX,'angle_diff');
  % mappedX2 = abs(mappedX2); % 777
else
  mappedX2 = transform_vector(mappedX,'angle');
end

numClusters = N_DICT_SIZE;
[means2, covariances2, priors2] = vl_gmm(mappedX2', numClusters, 'verbose');


dict.means_mag = means1; dict.means_ang = means2;
dict.covariances_mag = covariances1; dict.covariances_ang = covariances2;
dict.priors_mag = priors1; dict.priors_ang = priors2;
dict.N_PATCH_SIZE = N_PATCH_SIZE;
dict.N_DICT_SIZE = N_DICT_SIZE;
dict.N_CLASS = N_CLASS;
dict.N_INSTANCE = N_INSTANCE;
dict.N_PATCHES = N_PATCHES;
dict.USE_DIFF_PATCH = useDiffSeq;